/***********************************************************/
/*                                                         */
/* Actor.cpp                                               */
/*                                                         */
/* created 25.08.2002                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#include "Actor.h"
#include "Model.h"
#include "D3DStructures.h"
#include "TextureManager.h"
#include "IMath.h"
#include "MathStructures.h"
#include "TextureManager.h"

// www.infinitecode.com game library
namespace IGAME
{

void Actor::Create(ActorSpecs *Specs)
{
	Actor::Specs = *Specs;
	Angle = (rand() % 16384) / 16384.0f * 2 * PI;
	Pos = Specs->Pos;
	ScreenPos = Specs->DS->ProjViewMtx * Pos;
	DS = Specs->DS;
	Specs->QT->AddRenderableModel(this);
	Mtx = new I3D::Matrix;
	//((I3D::Matrix *)Mtx)->SetTranslationMatrix(Pos);
	((I3D::Matrix *)Mtx)->SetTangentMatrix(Pos, Angle, Specs->QT->Specs.Surface->GetNormal(Pos.x, Pos.z));
	// overlay frame
	OverlayRS.AddVertex(I3D::Vertex(IMATH::Vector(-1, -1, 0), IMATH::Vector(0, 0, -1), IMATH::Vector2D(0, 0)));
	OverlayRS.AddVertex(I3D::Vertex(IMATH::Vector( 1, -1, 0), IMATH::Vector(0, 0, -1), IMATH::Vector2D(1, 0)));
	OverlayRS.AddVertex(I3D::Vertex(IMATH::Vector( 1,  1, 0), IMATH::Vector(0, 0, -1), IMATH::Vector2D(1, 1)));
	OverlayRS.AddVertex(I3D::Vertex(IMATH::Vector(-1,  1, 0), IMATH::Vector(0, 0, -1), IMATH::Vector2D(0, 1)));
	OverlayRS.CreateVBuf(Specs->DS->D3DDevice);
	OverlayRS.AddIndex(0);
	OverlayRS.AddIndex(1);
	OverlayRS.AddIndex(3);
	OverlayRS.AddIndex(1);
	OverlayRS.AddIndex(2);
	OverlayRS.AddIndex(3);
	OverlayRS.CreateIBuf(Specs->DS->D3DDevice);
	SelectionTexture = I3D::TextureManager::GetTexture(Specs->DS, "Textures\\Drag.bmp");
}

/*void Actor::Physics(I3D::D3DStruct *DS)
{
	ScreenPos = DS->ProjViewMtx * Pos;
}*/

bool Actor::Select(I3D::BoundingBox2D *BBox)
{
	ScreenPos = DS->ProjViewMtx * Pos;
	IMATH::Vector2D Scr(ScreenPos.x, ScreenPos.y);
	return BBox->IsIn(&Scr);
}

void Actor::RenderSelection(I3D::D3DStruct *DS)
{
	// alpha blending
	DS->D3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE);
	DS->D3DDevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_ONE);
	DS->D3DDevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_ONE);
	DS->D3DDevice->SetRenderState(D3DRS_ALPHATESTENABLE, TRUE);
	DS->D3DDevice->SetRenderState(D3DRS_ALPHAFUNC, D3DCMP_GREATER);
	DS->D3DDevice->SetRenderState(D3DRS_ALPHAREF, 0x10);
	DS->D3DDevice->SetRenderState(D3DRS_LIGHTING, FALSE);
	// view and projection matrices
	D3DMATRIX Matrix = {1,0,0,0,  0,1,0,0,  0,0,1,0,  0,0,0,1};
	DS->D3DDevice->SetTransform(D3DTS_VIEW, &Matrix);
	DS->D3DDevice->SetTransform(D3DTS_PROJECTION, &Matrix);
	DS->D3DDevice->SetTexture(0, SelectionTexture);
	// get position on screen
	IMATH::Vector ViewPos = DS->ViewMtx * Pos;
	ScreenPos = DS->ProjMtx * ViewPos;
//	if(ScreenPos.z <= 1 && ScreenPos.z >= 0)
	if(ViewPos.z >= 0)
	{
		// world matrix
		D3DMATRIX WMatrix = {10.0f/ViewPos.z,0,0,0,  0,10.0f/ViewPos.z,0,0,  0,0,1,0,  ScreenPos.x,ScreenPos.y,0,1};
		DS->D3DDevice->SetTransform(D3DTS_WORLD, &WMatrix);
		// render
		OverlayRS.RenderIndex(DS->D3DDevice);
		// reset world matrix
		DS->D3DDevice->SetTransform(D3DTS_WORLD, &Matrix);
	}
	// reset matrices
	DS->D3DDevice->SetTransform(D3DTS_VIEW, &DS->ViewMtx);
	DS->D3DDevice->SetTransform(D3DTS_PROJECTION, &DS->ProjMtx);
	// reset alpha blending
	DS->D3DDevice->SetRenderState(D3DRS_LIGHTING, TRUE);
	DS->D3DDevice->SetRenderState(D3DRS_ALPHATESTENABLE, FALSE);
	DS->D3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, FALSE);
}

}
