/***********************************************************/
/*                                                         */
/* Actor.h                                                 */
/*                                                         */
/* created 25.08.2002                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#ifndef ACTOR_H
#define ACTOR_H

#include "RenderableModel.h"
#include "MathStructures.h"
#include "D3DStruct.h"
#include "QuadTree.h"
#include "PhysicsStruct.h"
#include "RenderStruct.h"

// www.infinitecode.com game library
namespace IGAME
{

struct ActorSpecs
{
	I3D::D3DStruct *DS;
	IMATH::Vector Pos;
	I3D::QuadTree *QT;
};

class Actor: public I3D::RenderableModel
{
	I3D::RenderStruct<I3D::Vertex, I3D_FVFVERTEX, 16, 16> OverlayRS;

protected:
	ActorSpecs Specs;
	float Angle;
	IMATH::Vector ScreenPos;
	LPDIRECT3DTEXTURE8 SelectionTexture;
	I3D::D3DStruct *DS;

public:
	Actor(void) {}

	~Actor(void)
	{
		if(Mtx) delete Mtx;
	}

	virtual void Create(ActorSpecs *Specs);
	// Physics should be abstract to avoid too many function calls
	// (this function is highly speed relevant, because it is called
	// every frame for every actor)
	virtual void Physics(I3D::D3DStruct *DS) = 0;

	bool Select(I3D::BoundingBox2D *BBox);
	void RenderSelection(I3D::D3DStruct *DS);
	virtual void SetTargetPos(IMATH::Vector2DXZ Target) = 0;
};

}

#endif
