/***********************************************************/
/*                                                         */
/* ClientSocket.cpp                                        */
/*                                                         */
/* created 03.08.2002                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#include "ClientSocket.h"

// www.infinitecode.com Network Library
namespace INETW
{

DWORD WINAPI CSThreadProcedure(void *Parameter)
{
	ClientSocket *CS = (ClientSocket *)Parameter;
	CS->Available = 0;
	while(CS->ThreadRunning)
	{
		if(!CS->Available)
		{
			CS->Available = recv(CS->Sck, CS->NBBuf, CLIENTSOCKET_BUFSIZE, 0);
		}
		else
			Sleep(50);
	}
	return 0;
}

ClientSocket::ClientSocket(char *Server, int Port, bool Blocking)
	:Available(0)
{
	if(!Create(Server, Port, Blocking))
		throw NetworkException("Can't create socket");
}

ClientSocket::ClientSocket(int Sck, bool Blocking)
{
	ClientSocket::Sck = Sck;
	ClientSocket::Blocking = true;
	if(Sck)
	{
		ClientSocket::Blocking = Blocking;
		Available = 0;
		// non blocking
		if(!Blocking)
		{
			ThreadRunning = true;
			ThreadHandle = CreateThread(NULL, 0, CSThreadProcedure, this, 0, &ThreadId);
		}
	}
	else
	{
		throw NetworkException("Can't create socket");
	}
}

ClientSocket::~ClientSocket(void)
{
	if(Sck)
		Destroy();
}

bool ClientSocket::Create(char *Server, int Port, bool Blocking)
{
	TCPSocket::Create();

	ClientSocket::Blocking = Blocking;
	struct sockaddr_in Adr;
	struct hostent *he;
	memset((char *)&Adr,0,sizeof(Adr)); 
	Adr.sin_family = AF_INET;
	Adr.sin_port = htons(Port);
	if ((he=gethostbyname(Server)) == NULL)
		return false;
	Adr.sin_addr = *((struct in_addr *)he->h_addr);
	memset(&(Adr.sin_zero), 0, 8);
	if(connect(Sck, (struct sockaddr *)&Adr, sizeof(struct sockaddr))!=0) return false;
	Available = 0;
	// non blocking
	if(!Blocking)
	{
		ThreadRunning = true;
		ThreadHandle = CreateThread(NULL, 0, CSThreadProcedure, this, 0, &ThreadId);
	}
	return true;
}

int ClientSocket::Recv(char *Buf, int Len)
{
	int BufLen = min(CLIENTSOCKET_BUFSIZE, Len);
	if(Blocking)
	{
		return recv(Sck, Buf, Len, 0);
	}
	else
	{
		if(Available)
		{
			memcpy(Buf, NBBuf, BufLen);
			if(Available > BufLen)
			{
				int Num = Available - BufLen;
				for(int i = 0; i < Num; i++)
					NBBuf[i] = NBBuf[i + BufLen];
				Available -= BufLen;
				return BufLen;
			}
			else
			{
				int Ret = Available;
				Available = 0;
				return Ret;
			}
		}
		else
			return 0;
	}
}

int ClientSocket::RecvPeek(char *Buf)
{
	if(Blocking)
	{
		return recv(Sck, Buf, CLIENTSOCKET_BUFSIZE, MSG_PEEK);
	}
	else
	{
		if(Available)
		{
			memcpy(Buf, NBBuf, CLIENTSOCKET_BUFSIZE);
			return Available;
		}
		else
			return 0;
	}
}

int ClientSocket::Send(char *Buf, int Len)
{
	return send(Sck, Buf, Len, 0);
}

void ClientSocket::Destroy(void)
{
	if(!Blocking)
	{
		ThreadRunning = false;
		TerminateThread(ThreadHandle, 0);
	}
	TCPSocket::Destroy();
}

}
