/***********************************************************/
/*                                                         */
/* D3DWindow.h                                             */
/*                                                         */
/* created 13.07.2002                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#ifndef D3DWINDOW_H
#define D3DWINDOW_H

#include "PrefixException.h"
#include "HashTable.h"
#include "D3DException.h"
#include "EventSender.h"

#define WIN32_LEAN_AND_MEAN // don't include rarely used header parts
#include <windows.h>

#include <d3d8.h>

// www.infinitecode.com 3d library
namespace I3D
{

#define DBG_D3D	"Direct3D.txt"

#define RELEASE(x) if(x){(x)->Release();}

LRESULT CALLBACK WndProc(HWND HWnd, UINT Message, WPARAM WParam, LPARAM LParam);

// event

// event id
enum
{
	D3DW_EVENT_RESET = 1,
	D3DW_EVENT_KEYDOWN = 2,
	D3DW_EVENT_WINDOWSEVENT = 3
};

// this class is passed by value, so keep it small or change to pass by reference
class D3DWEvent
{
	int ID;
	int Info, WParam, LParam;

public:
	D3DWEvent(void)
	{
		D3DWEvent::ID = 0;
	}

	D3DWEvent(int ID)
	{
		D3DWEvent::ID = ID;
	}

	D3DWEvent(int ID, int Info)
	{
		D3DWEvent::ID = ID;
		D3DWEvent::Info = Info;
	}

	D3DWEvent(int ID, int Info, int WParam, int LParam)
	{
		D3DWEvent::ID = ID;
		D3DWEvent::Info = Info;
		D3DWEvent::WParam = WParam;
		D3DWEvent::LParam = LParam;
	}

	int GetID(void)
	{
		return ID;
	}

	int GetInfo(void)
	{
		return Info;
	}

	int GetWParam(void)
	{
		return WParam;
	}

	int GetLParam(void)
	{
		return LParam;
	}
};

class D3DWindow: public IEV::EventSender<D3DWEvent>
{
	// Event handling
	friend LRESULT CALLBACK D3DW_WndProc(HWND HWnd, UINT Message, WPARAM WParam, LPARAM LParam);

	struct WindowPosition
	{
		int WindowStyle;
		int Clientdx, Clientdy;
	};

	WNDCLASS WinCl;
	HWND HWnd;
	HINSTANCE Instance;
	WindowPosition Pos, DesiredPos;
	char WndClassName[80];

	// Direct3D
	LPDIRECT3D8 D3D;
	D3DPRESENT_PARAMETERS D3DPP;

	// Event handling
	virtual LRESULT CALLBACK OnWindowMessage(UINT Message, WPARAM WParam, LPARAM LParam);

	// Window (with DW to avoid collisions with the Windows API functions CreateWindow and DestroyWindow) (DW = D3DWindow)
	void DWCreateWindow(int WindowStyle, char *Caption, int dx, int dy);
	void DWDestroyWindow(void);
	// Direct3D (with DW to avoid collisions with Direct3D API functions)
	void DWCreateDirect3D(void);
	void DWDestroyDirect3D(void);
	void ResetDirect3D(void);

public:
	class D3DWException: public D3DException
	{
	public:
		D3DWException(char *Description)
			:D3DException(Description)
		{
			AddPrefix("D3D Window: ");
		}
	};

	// lookup table for HWND and D3DWindow
	static ISL::HashTable<HWND, D3DWindow *, 10, 10> HWndTable;

	LPDIRECT3DDEVICE8 D3DDevice;

	D3DWindow(void);
	~D3DWindow(void);

	void Create(int WindowStyle, char *Caption, int dx, int dy);
	void Destroy(void);

	HWND GetHWnd(void)
	{
		return HWnd;
	}

	int GetWidth(void)
	{
		return Pos.Clientdx;
	}

	int GetHeight(void)
	{
		return Pos.Clientdy;
	}
};

}

#endif
