/***********************************************************/
/*                                                         */
/* IniLoader.cpp                                           */
/*                                                         */
/* created 05.08.2002                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#include "IniLoader.h"

#include <fstream.h>

namespace ITL
{

IniLoader::Group::Group(void)
:Name(NULL)
{
}

IniLoader::Group::Group(const char *Name)
{
	Create(Name);
}

IniLoader::Group::Create(const char *Name)
{
	Group::Name = new char[strlen(Name) + 1];
	strcpy(Group::Name, Name);
}

IniLoader::Group::~Group(void)
{
	delete[] Name;
}

void IniLoader::Group::Add(const char *Key, const char *Value)
{
	Table.Add(Key, Value);
}

//#include <stdio.h>

IniLoader::IniLoader(const char *FileName)
{
	ifstream in(FileName);
//	FILE *inf = fopen(FileName, "rt");
	Group *CurrentGroup = NULL;
	char Str[1024];
	while(!in.eof())
//	while(!feof(inf))
	{
		in.getline(Str, 1024, '\n');
		//fgets(Str, 1024, inf);
		// sort out inline comments
		char *s = Str;
		while(*s && *s != ';')
			s++;
		*s = 0;
		switch(Str[0])
		{
		case '[':
			{
				// new group
				char *s = Str;
				while(*s && *s != ']')
					s++;
				if(*s == ']')
				{
					*s = 0;
					s = Str + 1;
					CurrentGroup = NULL;
					for(int i = 0; i < Groups.Count(); i++)
					{
						if(!strcmpi(Groups[i].Name, s))
							CurrentGroup = &Groups[i];
					}
					if(!CurrentGroup)
					{
						Groups[Groups.Count()] = ITL::IniLoader::Group(s);
						CurrentGroup = &Groups[Groups.Count() - 1];
					}
				}
				//else ignore
				break;
			}
		case ';':
			// comment
			cout << "comment: " << Str << endl;
			break;
		case 0:
			// empty line
			break;
		default:
			// line
			char *Value = Str;
			while(*Value && *Value != '=')
				Value++;
			if(*Value == '=')
			{
				*Value = 0;
				Value++;
				if(!CurrentGroup)
				{
					Groups[Groups.Count()] = ITL::IniLoader::Group("");
					CurrentGroup = &Groups[Groups.Count() - 1];
				}
//				char *NewKey, *NewValue;
				char *Key = Str;
				while((*Key == ' ' || *Key == '\t') && *Key) Key++;
				while(Key[strlen(Key) - 1] == ' ' || Key[strlen(Key) - 1] == '\t') Key[strlen(Key) - 1] = 0;
				while((*Value == ' ' || *Value == '\t') && *Value) Value++;
				while(Value[strlen(Value) - 1] == ' ' || Value[strlen(Value) - 1] == '\t') Value[strlen(Value) - 1] = 0;
				CurrentGroup->Add(Str, Value);
			}
			// else ignore
			break;
		}
	}
	//fclose(inf);
}

const char *IniLoader::GetValue(const char *Group, const char *Key)
{
	for(int i = 0; i < Groups.Count(); i++)
	{
		if(!strcmpi(Groups[i].Name, Group))
		{
			ISL::String *Str = Groups[i].Table.GetValuePtr(ISL::String(Key));
			if(Str)
				return Str->GetChars();
			else
				return NULL;
		}
	}
	return NULL;
}

const char *IniLoader::GetKey(const char *Group, int Num)
{
	for(int i = 0; i < Groups.Count(); i++)
	{
		if(!strcmpi(Groups[i].Name, Group))
		{
			const char *Ret = Groups[i].Table.GetKeyPtr(Num)->GetChars();
			return Ret;
		}
	}
	return NULL;
}

int IniLoader::CountGroups(void)
{
	return Groups.Count();
}

int IniLoader::CountKeys(const char *Group)
{
	for(int i = 0; i < Groups.Count(); i++)
	{
		if(!strcmpi(Groups[i].Name, Group))
		return Groups[i].Table.CountKeys();
	}
	return 0;	
}

}
