/***********************************************************/
/*                                                         */
/* MathStructures.h                                        */
/*                                                         */
/* created 22.07.2002                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#ifndef MATHSTRUCTURES_H
#define MATHSTRUCTURES_H

#include <math.h>

// www.infinitecode.com math library
namespace IMATH
{

class Vector2D
{
public:
	union
	{
		struct
		{
			float x, y;
		};
		float Data[2];
	};

	Vector2D() {}

	Vector2D(float x, float y)
	{
		this->x = x;
		this->y = y;
	}
};

class Vector2DXZ
{
public:
	union
	{
		struct
		{
			float x, z;
		};
		float Data[2];
	};

	Vector2DXZ() {}

	Vector2DXZ(float x, float z)
	{
		this->x = x;
		this->z = z;
	}

	bool operator == (Vector2DXZ V)
	{
		return x == V.x && z == V.z;
	}

	Vector2DXZ operator - (Vector2DXZ V)
	{
		return Vector2DXZ(x - V.x, z - V.z);
	}
};

class Vector
{
public:
	union
	{
		struct
		{
			float x, y, z;
		};
		float Data[3];
	};

	Vector() {}

	Vector(float x, float y, float z)
	{
		this->x = x;
		this->y = y;
		this->z = z;
	}

	float Length(void)
	{
		return (float)sqrt(x * x + y * y + z * z);
	}

	float LengthSq(void)
	{
		return x * x + y * y + z * z;
	}

	static Vector CrossProduct(Vector V1, Vector V2)
	{
		return Vector(V1.y * V2.z - V1.z * V2.y, V1.z * V2.x - V1.x * V2.z, V1.x * V2.y - V1.y * V2.x);
	}

	static Vector Normalize(Vector V)
	{
		float OOLen = 1/(float)sqrt(V.x * V.x + V.y * V.y + V.z * V.z);
		return Vector(OOLen * V.x, OOLen * V.y, OOLen * V.z);
	}

	Vector operator + (const Vector& V2) const
	{
		return Vector(x + V2.x, y + V2.y, z + V2.z);
	}

	Vector operator - (const Vector& V2) const
	{
		return Vector(x - V2.x, y - V2.y, z - V2.z);
	}

	Vector operator += (const Vector& V2)
	{
		x += V2.x;
		y += V2.y;
		z += V2.z;
		return *this;
	}

	Vector operator - () const
	{
		return Vector(-x, -y, -z);
	}

	bool operator == (const Vector V2) const
	{
		if(x == V2.x && y == V2.y && z == V2.z)
			return true;
		else
			return false;
	}

	friend Vector operator * (float Scalar, Vector& V);
};

// multiplication with a scalar value
Vector operator * (float Scalar, Vector& V);

}

#endif
