/***********************************************************/
/*                                                         */
/* Model.cpp                                               */
/*                                                         */
/* created 20.08.2002                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#include "Model.h"
#include "IniLoader.h"

// www.infinitecode.com 3d library
namespace I3D
{

ISL::DynamicArray<Model::LoadedModel, 0, 1> Model::LoadedModels;

Model::Model(void)
{
}

void Model::Create(D3DStruct *DS, char *FileName)
{
	ITL::IniLoader Loader(FileName);
	int Vertices = Loader.CountKeys("Vertices");
	int i;
	for(i = 0; i < Vertices; i++)
	{
		char Str[1024];
		char Value[1024];
		Vertex V;
		strcpy(Value, Loader.GetValue("Vertices", itoa(i, Str, 10)));
		// parse
		char *Srch = Value;
		while((*Srch == ' ' || *Srch == '\t') && Srch != 0) Srch++;
		V.x = (float)atof(Srch);
		while(*Srch != ',' && *Srch) Srch++;
		Srch++;
		while((*Srch == ' ' || *Srch == '\t') && Srch != 0) Srch++;
		V.y = (float)atof(Srch);
		while(*Srch != ',' && *Srch) Srch++;
		Srch++;
		while((*Srch == ' ' || *Srch == '\t') && Srch != 0) Srch++;
		V.z = (float)atof(Srch);
		while(*Srch != ',' && *Srch) Srch++;
		Srch++;
		while((*Srch == ' ' || *Srch == '\t') && Srch != 0) Srch++;
		V.tu = (float)atof(Srch);
		while(*Srch != ',' && *Srch) Srch++;
		Srch++;
		while((*Srch == ' ' || *Srch == '\t') && Srch != 0) Srch++;
		V.tv = (float)atof(Srch);
		// add vertex to renderstruct
		RS.AddVertex(V);
	}
	int Triangles = Loader.CountKeys("Triangles");
	for(i = 0; i < Triangles; i++)
	{
		char Str[1024];
		char Value[1024];
		strcpy(Value, Loader.GetValue("Triangles", itoa(i, Str, 10)));
		// parse
		char *Srch = Value;
		while((*Srch == ' ' || *Srch == '\t') && Srch != 0) Srch++;
		int Index1 = atoi(Srch);
		while(*Srch != ',' && *Srch) Srch++;
		Srch++;
		while((*Srch == ' ' || *Srch == '\t') && Srch != 0) Srch++;
		int Index2 = atoi(Srch);
		while(*Srch != ',' && *Srch) Srch++;
		Srch++;
		while((*Srch == ' ' || *Srch == '\t') && Srch != 0) Srch++;
		int Index3 = atoi(Srch);
		// add indices to renderstruct
		RS.AddIndex(Index1);
		RS.AddIndex(Index2);
		RS.AddIndex(Index3);
		// calculate normal
		Vector Vector1 = RS.RenderList[Index1].v;
		Vector Vector2 = RS.RenderList[Index2].v;
		Vector Vector3 = RS.RenderList[Index3].v;
		Vector Cross1 = Vector1 - Vector2;
		Vector Cross2 = Vector3 - Vector2;
		Vector Norm = Vector::Normalize(Vector::CrossProduct(Cross1, Cross2));
		// assign normal to vertices
		RS.RenderList[Index1].n = Norm;
		RS.RenderList[Index2].n = Norm;
		RS.RenderList[Index3].n = Norm;
	}
	RS.CreateVBuf(DS->D3DDevice); // after normal calculations!
	RS.CreateIBuf(DS->D3DDevice);
	const char *StdTexture = Loader.GetValue("Texture", "Name");
	if(StdTexture)
		strcpy(StandardTexture, StdTexture);
	else
		strcpy(StandardTexture, "");
}

void Model::RenderSetup(D3DStruct *DS)
{
	RS.SetupIndex(DS->D3DDevice);
}

}
