/***********************************************************/
/*                                                         */
/* Model.h                                                 */
/*                                                         */
/* created 20.08.2002                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#ifndef MODEL_H
#define MODEL_H

#include "D3DStruct.h"
#include "Renderable.h"

// www.infinitecode.com 3d library
namespace I3D
{

class Model: public RenderableIndex
{
protected:
	class LoadedModel
	{
	public:
		LoadedModel(void)
			:Name(0)
		{
		}

		LoadedModel(const char *Name, Model *Mdl)
		{
			LoadedModel::Name = new char[1024];
			strcpy(LoadedModel::Name, Name);
			LoadedModel::Mdl = Mdl;
		}

		~LoadedModel(void)
		{
			if(Name) delete[] Name;
		}

		LoadedModel& operator = (LoadedModel &LM)
		{
			if(!Name)
				Name = new char[1024];
			strcpy(Name, LM.Name);
			Mdl = LM.Mdl;
			return *this;
		}

		LoadedModel(LoadedModel& LM)
		{
			*this = LM;
		}

		char *Name;
		Model *Mdl;

	};

	static ISL::DynamicArray<LoadedModel, 0, 1> LoadedModels;

public:
	char StandardTexture[1024];

	Model(void);

	static Model *GetModel(D3DStruct *DS, char *Name)
	{
		for(int i = 0; i < LoadedModels.Count(); i++)
			if(!strcmpi(Name, LoadedModels[i].Name))
				return LoadedModels[i].Mdl;
		Model *NewModel = new Model();
		/*char FileName[1024];
		strcpy(FileName, "Models\\");
		strcat(FileName, Name);
		strcat(FileName, ".t4m");*/
		NewModel->Create(DS, Name);
		LoadedModels[LoadedModels.Count()] = LoadedModel(Name, NewModel);
		return NewModel;
	}

	void Create(D3DStruct *DS, char *FileName);
	virtual void CreateView(D3DStruct *DS) {}
	//virtual void Render(D3DStruct *DS); // uses RenderableIndex::Render
	void RenderSetup(D3DStruct *DS);
	virtual void PreRender(D3DStruct *DS) {}
	virtual void PostRender(D3DStruct *DS) {}

	const char *GetStandardTexture()
	{
		return StandardTexture;
	}
};

}

#endif
