/***********************************************************/
/*                                                         */
/* MouseCursor.h                                           */
/*                                                         */
/* created 26.08.2002                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#ifndef MOUSECURSOR_H
#define MOUSECURSOR_H

#include "D3DStruct.h"
#include "Input.h"
#include "DynamicArray.h"
#include "RenderStruct.h"

// www.infinitecode.com 3d library
namespace I3D
{

enum
{
	MC_CLICK =		1, // cancels drag
	MC_DRAGSTART =	2,
	MC_DRAGEND =	3
};

class MouseCursor
{
public:
	class MouseCursorEvent
	{
	public:
		int Code;
		float x, y, x2, y2;
	};

private:
	IIN::Input In;
	HWND Wnd;
	LPDIRECT3DTEXTURE8 Texture, DragTexture;
	RECT ClientRect;
	float x, y;
	int Buttons;
	float Downx, Downy;
	int IDownx, IDowny;
	bool Drag;
	ISL::DynamicArray<MouseCursorEvent, 0, 1> Events;
	RenderStruct<Vertex, I3D_FVFVERTEX, 16, 16> RS;

	void CreateView(D3DStruct *DS);

public:

	void Create(D3DStruct *DS, HWND Wnd);
	bool GetEvent(MouseCursorEvent *Event);

	void Render(D3DStruct *DS);

	float GetX(void)
	{
		return x;
	}

	float GetY(void)
	{
		return y;
	}

	float GetZ(void)
	{
		return (float)In.GetZ();
	}
};

}

#endif
