/***********************************************************/
/*                                                         */
/* NURBS.cpp                                               */
/*                                                         */
/* created 21.07.2002                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

// non uniform rational b-spline

#include "NURBS.h"

// TODO: delete if not needed anymore
#include <stdio.h>

// www.infinitecode.com 3d library
namespace I3D
{

NURBS::NURBS(void)
{
	HeightMap = NULL;
}

NURBS::~NURBS(void)
{
	if(HeightMap) delete HeightMap;
}

float NURBS::GetY(float x, float z)
{
	// TODO: implement b-spline
	// TODO: optimize calculations
	x /= Size * 0.5f;
	z /= Size * 0.5f;
	x += 1.0f;
	z += 1.0f;
	x *= XRes * 0.5f;
	z *= ZRes * 0.5f;
	if(x < 0.0f) x = 0.0f;
	if(x > XRes - 2) x = (float)XRes - 2;
	if(z < 0.0f) z = 0.0f;
	if(z > ZRes - 2) z = (float)ZRes - 2;
	float XFac = x - int(x);
	float ZFac = z - int(z);
	float XInt1 = (1 - XFac) * HeightMap[(int)x + (int)z * XRes] + XFac * HeightMap[(int)x + 1 + (int)z * XRes];
	float XInt2 = (1 - XFac) * HeightMap[(int)x + ((int)z + 1) * XRes] + XFac * HeightMap[(int)x + 1 + ((int)z + 1) * XRes];
	return ((1 - ZFac) * XInt1 + ZFac * XInt2);
}

Vector NURBS::GetNormal(float x, float z)
{
	float Delta = 3.0f;
	Vector V1 = Vector(x + Delta, GetY(x + Delta, z), z) - Vector(x - Delta, GetY(x - Delta, z), z);
	Vector V2 = Vector(x, GetY(x, z + Delta), z + Delta) - Vector(x, GetY(x, z - Delta), z - Delta);
	Vector CP = -Vector::CrossProduct(V1, V2);
	Vector NCP = Vector::Normalize(CP);
	return NCP;
}

void NURBS::Create(int XRes, int ZRes, float Size)
{
	NURBS::Size = Size;
	NURBS::XRes = XRes;
	NURBS::ZRes = ZRes;
	HeightMap = (float *)malloc(XRes * ZRes * sizeof(float));
	// TODO: load or generate HeightMap from mission
	char fname[1024];
	sprintf(fname, "HeightMap%dx%d.bmp", XRes, ZRes);
	FILE *f = fopen(fname, "rb");
	fseek(f, -XRes*ZRes*3, SEEK_END);
//	for(int j = ZRes - 1; j >= 0; j--)
	for(int j = 0; j < ZRes; j++)
		for(int i = 0; i < XRes; i++)
			HeightMap[i + j * XRes] = (fgetc(f)/256.0f + fgetc(f)/256.0f + fgetc(f)/256.0f)/3.0f * 0.3f * (Size * 0.5f);
	fclose(f);
}

}
