/***********************************************************/
/*                                                         */
/* RenderStruct.h                                          */
/*                                                         */
/* created 20.07.2002                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#ifndef RENDERSTRUCT_H
#define RENDERSTRUCT_H

#include "DynamicArray.h"
#include "D3DStructures.h"

#include <d3d8.h>

// www.infinitecode.com 3d library
namespace I3D
{

template<class VtxType, int FVF, int StartSize = 256, int Increment = 256>
class RenderStruct
{
	LPDIRECT3DVERTEXBUFFER8 VBuf;
	LPDIRECT3DINDEXBUFFER8 IBuf;

public:
	ISL::DynamicArray<VtxType, StartSize, Increment> RenderList;
	ISL::DynamicArray<unsigned short, StartSize, Increment> IndexList;

	RenderStruct(void)
	{
		VBuf = NULL;
		IBuf = NULL;
	}

	~RenderStruct(void)
	{
		RELEASE(VBuf);
		RELEASE(IBuf);
	}

	/*RenderStruct& operator = (RenderStruct& RS)
	{
		VBuf = 0;
		IBuf = 0;
		RenderList = RS.RenderList;
		IndexList = RS.IndexList;
		return *this;
	}*/

	void AddVertex(VtxType Vtx)
	{
		RenderList[RenderList.Count()] = Vtx;
	}

	void AddIndex(unsigned short Index)
	{
		IndexList[IndexList.Count()] = Index;
	}

	void ClearVertex(void)
	{
		RenderList.Clear();
	}

	void ClearIndex(void)
	{
		IndexList.Clear();
	}

	int GetVertexNumber(void)
	{
		return RenderList.Count();
	}

	int GetIndexNumber(void)
	{
		return IndexList.Count();
	}

	void CreateVBuf(LPDIRECT3DDEVICE8 D3DDevice)
	{
		if(VBuf) RELEASE(VBuf);
		VBuf = NULL;
		if(RenderList.Count())
		{
			// create vertexbuffer
			D3DDevice->CreateVertexBuffer(RenderList.Count() * 3 * sizeof(VtxType), D3DUSAGE_WRITEONLY, I3D_FVFVERTEX, D3DPOOL_MANAGED, &VBuf);
			// fill vertexbuffer
			VtxType *Buf = NULL;
			VBuf->Lock(0, 0, (BYTE**)&Buf, 0 /*D3DLOCK_NOSYSLOCK*/);
			memcpy(Buf, RenderList.GetPtr(), RenderList.Count() * sizeof(VtxType));
			VBuf->Unlock();
		}
	}

	void CreateIBuf(LPDIRECT3DDEVICE8 D3DDevice)
	{
		if(IBuf) RELEASE(IBuf);
		IBuf = NULL;
		if(IndexList.Count())
		{
			// create vertexbuffer
			D3DDevice->CreateIndexBuffer(IndexList.Count() * 3 * sizeof(unsigned short), D3DUSAGE_WRITEONLY, D3DFMT_INDEX16, D3DPOOL_MANAGED, &IBuf);
			// fill indexbuffer
			unsigned short *Buf = NULL;
			IBuf->Lock(0, 0, (BYTE**)&Buf, 0 /*D3DLOCK_NOSYSLOCK*/);
			memcpy(Buf, IndexList.GetPtr(), IndexList.Count() * sizeof(unsigned short));
			IBuf->Unlock();
		}
	}

	void SetupVertex(LPDIRECT3DDEVICE8 D3DDevice)
	{
		if(VBuf)
		{
			D3DDevice->SetVertexShader(FVF);
			D3DDevice->SetStreamSource(0, VBuf, sizeof(VtxType));
		}
	}

	void RenderVertex(LPDIRECT3DDEVICE8 D3DDevice)
	{
		if(VBuf)
		{
			D3DDevice->SetVertexShader(FVF);
			D3DDevice->SetStreamSource(0, VBuf, sizeof(VtxType));
			D3DDevice->DrawPrimitive(D3DPT_TRIANGLELIST, 0, GetVertexNumber() / 3);
		}
	}

	void RenderIndex(LPDIRECT3DDEVICE8 D3DDevice)
	{
		if(VBuf && IBuf)
		{
			D3DDevice->SetVertexShader(FVF);
			D3DDevice->SetStreamSource(0, VBuf, sizeof(VtxType));
			D3DDevice->SetIndices(IBuf, 0);
			D3DDevice->DrawIndexedPrimitive(D3DPT_TRIANGLELIST, 0, GetVertexNumber(), 0, GetIndexNumber() / 3);
		}
	}

	void RenderIndex(LPDIRECT3DDEVICE8 D3DDevice, int VertexOffset, int VertexNumber, int IndexOffset, int IndexNumber)
	{
		if(VBuf && IBuf)
		{
			D3DDevice->SetVertexShader(FVF);
			D3DDevice->SetStreamSource(0, VBuf, sizeof(VtxType));
			D3DDevice->SetIndices(IBuf, VertexOffset);
			D3DDevice->DrawIndexedPrimitive(D3DPT_TRIANGLELIST, 0, VertexNumber, IndexOffset, IndexNumber / 3);
		}
	}

	// just prepare for rendering
	bool SetupIndex(LPDIRECT3DDEVICE8 D3DDevice)
	{
		if(VBuf && IBuf)
		{
			D3DDevice->SetVertexShader(FVF);
			D3DDevice->SetStreamSource(0, VBuf, sizeof(VtxType));
			D3DDevice->SetIndices(IBuf, 0);
			return true;
		}
		else
			return false;
	}

};

}

#endif
