/***********************************************************/
/*                                                         */
/* Renderable.cpp                                          */
/*                                                         */
/* created 29.07.2002                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#include "Renderable.h"

// www.infinitecode.com 3d library
namespace I3D
{

Renderable::Renderable(void)
:RenderFlags(0)
{
	OnViewChange();
}

// called when the camera moves
void Renderable::OnViewChange(void)
{
	RenderFlags |= RENDERABLE_FLAG_VIEWCHANGE;
}

void Renderable::Render(D3DStruct *DS)
{
	if(RenderFlags & RENDERABLE_FLAG_VIEWCHANGE)
	{
		RenderFlags &= ~RENDERABLE_FLAG_VIEWCHANGE;
		// create new View
		CreateView(DS);
	}
}

// ****** RenderableVertex ******

void RenderableVertex::Render(D3DStruct *DS)
{
	if(RenderFlags & RENDERABLE_FLAG_VIEWCHANGE)
	{
		RenderFlags &= ~RENDERABLE_FLAG_VIEWCHANGE;
		// create new View
		CreateView(DS);
	}
	RS.RenderVertex(DS->D3DDevice);
}

// ****** RenderableIndex ******

void RenderableIndex::Render(D3DStruct *DS)
{
	if(RenderFlags & RENDERABLE_FLAG_VIEWCHANGE)
	{
		RenderFlags &= ~RENDERABLE_FLAG_VIEWCHANGE;
		// create new View
		CreateView(DS);
	}
	RS.RenderIndex(DS->D3DDevice);
}

// ****** RenderableIndexTex2 ******

void RenderableIndexTex2::Render(D3DStruct *DS)
{
	if(RenderFlags & RENDERABLE_FLAG_VIEWCHANGE)
	{
		RenderFlags &= ~RENDERABLE_FLAG_VIEWCHANGE;
		// create new View
		CreateView(DS);
	}
	RS.RenderIndex(DS->D3DDevice);
}

}
