/***********************************************************/
/*                                                         */
/* Renderable.h                                            */
/*                                                         */
/* created 29.07.2002                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#ifndef RENDERABLE_H
#define RENDERABLE_H

#include "D3DStruct.h"
#include "RenderStruct.h"
#include "D3DStructures.h"

// www.infinitecode.com 3d library
namespace I3D
{

// RenderFlags
#define RENDERABLE_FLAG_VIEWCHANGE		1

class Renderable
{
protected:
	int RenderFlags;

	virtual void CreateView(D3DStruct *DS) = 0;

public:
	Renderable(void);

	virtual void Render(D3DStruct *DS);
	// called when the camera moves, creates an indexbuffer
	void OnViewChange(void);
};

class RenderableVertex: public Renderable
{
protected:
	virtual void CreateView(D3DStruct *DS) = 0;

public:
	RenderStruct<Vertex, I3D_FVFVERTEX, 16, 16> RS;

	virtual void Render(D3DStruct *DS);
};

class RenderableIndex: public Renderable
{
protected:
	virtual void CreateView(D3DStruct *DS) = 0;

public:
	RenderStruct<Vertex, I3D_FVFVERTEX, 16, 16> RS;

	virtual void Render(D3DStruct *DS);
};

class RenderableIndexTex2: public Renderable
{
protected:
	virtual void CreateView(D3DStruct *DS) = 0;

public:
	RenderStruct<VertexTex2, I3D_FVFVERTEXTEX2, 16, 16> RS;

	virtual void Render(D3DStruct *DS);
};

}

#endif
