/***********************************************************/
/*                                                         */
/* ServerSocket.cpp                                        */
/*                                                         */
/* created 03.08.2002                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#include "ServerSocket.h"

// www.infinitecode.com Network Library
namespace INETW
{

DWORD WINAPI SSThreadProcedure(void *Parameter)
{
	ServerSocket *SS = (ServerSocket *)Parameter;
	while(SS->ThreadRunning)
	{
		if(SS->CSck == 0)
		{
			int NewSocket = accept(SS->Sck, (struct sockaddr *)&SS->Adr, NULL);
			if(NewSocket > 0) SS->CSck = NewSocket;
		}
		else
			Sleep(50);
	}
	return 0;
}

ServerSocket::ServerSocket(char *Host, int Port, bool Blocking)
	:CSck(0), ThreadRunning(false)
{
	if(!Create(Host, Port, Blocking))
		throw NetworkException("Can't create server. Port already in use?");
}

ServerSocket::~ServerSocket(void)
{
	if(Sck)
		Destroy();
}

bool ServerSocket::Create(char *Host, int Port, bool Blocking)
{
	TCPSocket::Create();

	ServerSocket::Blocking = Blocking;
	struct hostent *he;
	memset((char *)&Adr,0,sizeof(Adr)); 
	Adr.sin_family = AF_INET;
	Adr.sin_port = htons(Port);
	if ((he=gethostbyname(Host)) == NULL)
		return false;
	Adr.sin_addr = *((struct in_addr *)he->h_addr);
	memset(&(Adr.sin_zero), 0, 8);
	if(bind(Sck, (struct sockaddr *)&Adr, sizeof(struct sockaddr))!=0) return false;
	if(listen(Sck, 1)!=0) return false;
	// non blocking
	if(!Blocking)
	{
		ThreadRunning = true;
		ThreadHandle = CreateThread(NULL, 0, SSThreadProcedure, this, 0, &ThreadId);
	}
	return true;
}

void ServerSocket::Destroy(void)
{
	if(!Blocking)
	{
		ThreadRunning = false;
		TerminateThread(ThreadHandle, 0);
	}
	TCPSocket::Destroy();
}

ClientSocket *ServerSocket::Accept(bool ClientBlocking)
{
	if(Blocking)
	{
		CSck = accept(Sck, (struct sockaddr *)&Adr, NULL);
		if(CSck)
		{
			return new ClientSocket(CSck, ClientBlocking);
		}
		else
		{
			return NULL;
		}
	}
	else
	{
		if(CSck)
		{
			int Ret = CSck;
			CSck = 0;
			return new ClientSocket(Ret, ClientBlocking);
		}
		else
			return NULL;
	}
}

}
