/***********************************************************/
/*                                                         */
/* Tank.cpp                                                */
/*                                                         */
/* created 27.08.2002                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#include "Tank.h"
#include "IMath.h"
#include "TextureManager.h"

// www.infinitecode.com game library
namespace IGAME
{

void Tank::Create(ActorSpecs *Specs)
{
	AIOrder = 0;
	Mdl = I3D::Model::GetModel(Specs->DS, "Models\\Tank.t4m");
	Texture = I3D::TextureManager::GetTexture(Specs->DS, "Textures\\Tank.bmp");
	Actor::Create(Specs);
	LastShot = Specs->DS->Sp->GameTime;
}

void Tank::Physics(I3D::D3DStruct *DS)
{
	//Actor::Physics(DS);
	if(!AIOrder)
	{
		Angle += 0.2f * DS->Sp->Delta;
	}
	else
	{
		MoveTo(Target);
	}
	Pos += I3D::Vector(16 * (float)sin(Angle) * DS->Sp->Delta, 0, 16 * (float)cos(Angle) * DS->Sp->Delta);
	// surface
	Pos.y = Specs.QT->Specs.Surface->GetY(Pos.x, Pos.z);
	((I3D::Matrix *)Mtx)->SetTangentMatrix(Pos, Angle, Specs.QT->Specs.Surface->GetNormal(Pos.x, Pos.z));
	Specs.QT->MoveRenderableModel(this);
	// shoot
	if(Specs.DS->Sp->GameTime - LastShot > 1)
	{
		LastShot = Specs.DS->Sp->GameTime;
		// spawn shot
		
	}
}

void Tank::SetTargetPos(IMATH::Vector2DXZ Target)
{
	Tank::Target = Target;
	AIOrder = 1;
}

void Tank::MoveTo(IMATH::Vector2DXZ Target)
{
	IMATH::Vector2DXZ Dir = Target - IMATH::Vector2DXZ(Pos.x, Pos.z);
	if(Dir.z)
	{
		float TargetAngle = (float)atan(Dir.x / Dir.z);
		if(Dir.z < 0) TargetAngle += PI;
		Angle = TargetAngle;
	}
}

}
