/***********************************************************/
/*                                                         */
/* Text.h                                                  */
/*                                                         */
/* created 29.08.2002                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#ifndef TEXT_H
#define TEXT_H

#include "D3DStruct.h"
#include "RenderStruct.h"
#include "TextureManager.h"

#include <d3d8.h>

// www.infinitecode.com 3d library
namespace I3D
{

class Text
{
	static LPDIRECT3DTEXTURE8 CourierNew;
	static RenderStruct<Vertex, I3D_FVFVERTEX, 256, 256> RS;

public:

	static void Text::Create(D3DStruct *DS)
	{
		CourierNew = TextureManager::GetTexture(DS, "Textures\\LucidaConsole.bmp");
		for(int i = 0; i < 256; i++)
		{
			float XOfs = (i & 0xF) / 16.0f;
			float YOfs = (i >> 4) / 16.0f;
			float Delta = 1 / 16.0f;
			RS.AddVertex(Vertex(Vector(0, 0, 0), Vector(0, 0, -1), Vector2D(XOfs, YOfs + Delta)));
			RS.AddVertex(Vertex(Vector(1, 0, 0), Vector(0, 0, -1), Vector2D(XOfs + Delta, YOfs + Delta)));
			RS.AddVertex(Vertex(Vector(1, 1, 0), Vector(0, 0, -1), Vector2D(XOfs + Delta, YOfs)));
			RS.AddVertex(Vertex(Vector(0, 1, 0), Vector(0, 0, -1), Vector2D(XOfs, YOfs)));
		}
		RS.CreateVBuf(DS->D3DDevice);
		RS.AddIndex(0);
		RS.AddIndex(1);
		RS.AddIndex(2);
		RS.AddIndex(0);
		RS.AddIndex(2);
		RS.AddIndex(3);
		RS.CreateIBuf(DS->D3DDevice);
	}

	static void Text::DrawChar(D3DStruct *DS, char c, float x, float y, float dx, float dy)
	{
		if(!CourierNew)
			Create(DS);
		Matrix Mtx, WMtx;
		Mtx.SetIdentityMatrix();
		WMtx.SetScaleTranslationMatrix(Vector(1.4f * dx, 1.4f * dy, 0), Vector(x - 0.2f * dx, y - 0.2f * dy, 0));
		DS->D3DDevice->SetTransform(D3DTS_VIEW, &Mtx);
		DS->D3DDevice->SetTransform(D3DTS_PROJECTION, &Mtx);
		DS->D3DDevice->SetTransform(D3DTS_WORLD, &WMtx);
		DS->D3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE);
		DS->D3DDevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_ONE);
		DS->D3DDevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_ONE);
		DS->D3DDevice->SetRenderState(D3DRS_ALPHATESTENABLE, TRUE);
		DS->D3DDevice->SetRenderState(D3DRS_ALPHAFUNC, D3DCMP_GREATER);
		DS->D3DDevice->SetRenderState(D3DRS_ALPHAREF, 0x10);
		DS->D3DDevice->SetRenderState(D3DRS_LIGHTING, FALSE);
		DS->D3DDevice->SetTexture(0, CourierNew);
		RS.RenderIndex(DS->D3DDevice, 4 * c, 4, 0, 6);
		DS->D3DDevice->SetRenderState(D3DRS_LIGHTING, TRUE);
		DS->D3DDevice->SetRenderState(D3DRS_ALPHATESTENABLE, FALSE);
		DS->D3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, FALSE);
		DS->D3DDevice->SetTransform(D3DTS_VIEW, &DS->ProjMtx);
		DS->D3DDevice->SetTransform(D3DTS_PROJECTION, &DS->ViewMtx);
	}

	static Text::DrawString(D3DStruct *DS, char *Str, float x, float y, float dx, float dy)
	{
		for(int i = 0; i < strlen(Str); i++)
		{
			DrawChar(DS, Str[i], x + i * dx, y, dx, dy);
		}
	}

};

}

#endif
