/***********************************************************/
/*                                                         */
/* Tree.cpp                                                */
/*                                                         */
/* created 16.07.2002                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#include "Tree.h"
#include "QuadTree.h"
#include "TextureManager.h"

// www.infinitecode.com game library
namespace IGAME
{

void Tree::Create(ActorSpecs *Specs)
{
	using namespace I3D;
	// position
	RenderableModel::Pos = Specs->Pos;
	// matrix
	Mtx = new Matrix();
	((Matrix *)Mtx)->SetTranslationMatrix(Pos);
	// texture
	Texture = TextureManager::GetTexture(Specs->DS, "Tree");
	// model
	Mdl = TreeModel::GetModel(Specs->DS, "Models\\Tree.t4m");
	Specs->QT->AddRenderableModel(this);
}

Tree::~Tree(void)
{
	if(Mtx)
		delete Mtx;
}

void Tree::Physics(I3D::D3DStruct *DS)
{
	//Actor::Physics(DS);
}

void Tree::SetTargetPos(IMATH::Vector2DXZ Target)
{
}

// ****** TreeModel ******

void TreeModel::PreRender(I3D::D3DStruct *DS)
{
	DS->D3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE);
	DS->D3DDevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
	DS->D3DDevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);
	DS->D3DDevice->SetRenderState(D3DRS_ALPHATESTENABLE, TRUE);
	DS->D3DDevice->SetRenderState(D3DRS_ALPHAFUNC, D3DCMP_GREATER);
	DS->D3DDevice->SetRenderState(D3DRS_ALPHAREF, 0x60);
	DS->D3DDevice->SetRenderState(D3DRS_CULLMODE,D3DCULL_NONE);
	//DS->D3DDevice->SetRenderState(D3DRS_LIGHTING,FALSE);
}

void TreeModel::PostRender(I3D::D3DStruct *DS)
{
	DS->D3DDevice->SetRenderState(D3DRS_CULLMODE,D3DCULL_CW);
	DS->D3DDevice->SetRenderState(D3DRS_ALPHATESTENABLE, FALSE);
	DS->D3DDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, FALSE);
	D3DMATRIX WMatrix = {1,0,0,0,  0,1,0,0,  0,0,1,0,  0,0,0,1};
    DS->D3DDevice->SetTransform(D3DTS_WORLD, &WMatrix);
}

}
