/***********************************************************/
/*                                                         */
/* World.h                                                 */
/*                                                         */
/* created 16.07.2002                                      */
/* by Christian Reimann  empgodot@gmx.de                   */
/*                                                         */
/* Copyright    Christian Reimann                          */
/*              empgodot@gmx.de                            */
/*              www.infinitecode.com                       */
/*                                                         */
/* This code may be freely modified and redistributed.     */
/* If you use parts of this code in a project,             */
/* I'd appreciate a mention in the credits.                */
/*                                                         */
/***********************************************************/

#ifndef WORLD_H
#define WORLD_H

#include "D3DStruct.h"
#include "QuadTree.h"
#include "Renderable.h"
#include "NURBS.h"
#include "DynamicArray.h"
#include "Tree.h"
#include "Actor.h"
#include "PhysicsStruct.h"

// www.infinitecode.com game library
namespace IGAME
{

#define GAME_TREES	1024

class World
{
	// world objects
	I3D::QuadTree QTree;
	//I3D::Forest For;
	I3D::NURBS Surface;
	ISL::DynamicArray<IGAME::Actor *, GAME_TREES, 256> StaticActors;
	ISL::DynamicArray<IGAME::Actor *, 0, 256> Actors;
	I3D::Vector CameraPos, CameraAngle;
	ISL::DynamicArray<Actor *, 0, 4> Selection;
	int RenderFlags;

	void CreateView(I3D::D3DStruct *DS);

public:
	~World(void)
	{
		Destroy();
	}

	virtual void Render(I3D::D3DStruct *DS);
	void Create(I3D::D3DStruct *DS);
	void Destroy(void);
	void Physics(I3D::D3DStruct *DS);
	void Init3D(I3D::D3DStruct *DS);
	void OnViewChange(void);

	void MoveCamera(I3D::Vector V);
	void MoveCameraDirect(I3D::Vector V);
	void RotateCamera(I3D::Vector V);
	void Select(I3D::BoundingBox2D *BBox);
	void Click(IMATH::Vector2D Pos);
};

}

#endif
